import os
import unittest
from unittest.mock import MagicMock, patch, mock_open
from pathlib import Path
import shutil
import pandas as pd
from apsimNGpy.tests.base_test import BaseTester, path
# Import the module where APSIMNG class is defined
from apsimNGpy.core.core import APSIMNG, save_model_to_file
from apsimNGpy.core.model_loader import save_model_to_file
from apsimNGpy.core.base_data import load_default_simulations
import config
from apsimNGpy.core.config import (set_apsim_bin_path, get_apsim_bin_path, scan_dir_for_bin, auto_detect_apsim_bin_path)

TEST_PATH = "/path/to/test/bin"  # apsim bin path to test

auto = auto_detect_apsim_bin_path()


class TestConfig(BaseTester):

    def test_get_apsim_bin_path(self):
        abp = get_apsim_bin_path()
        p_true = path.exists(abp)
        self.assertTrue(p_true)

    def test_set_apsim_bin_path(self):
        """
        Test that set_apsim_bin_path
        This will always fail if the TEST_PATH above is not pointing correctly to the correct path of APSIM bin files
        @return:
        """
        # Initially retrieve the current APSIM binary path
        original_path = get_apsim_bin_path()

        try:
            # Set a new test path

            set_apsim_bin_path(TEST_PATH, raise_errors=False)

            # Retrieve the path after setting it
            new_path = get_apsim_bin_path()

            # Check if the new path is as expected
            self.assertEqual(new_path, TEST_PATH, "The APSIM binary path was not updated correctly.")

        finally:
            # Clean up: restore an original path to avoid side effects in other tests
            set_apsim_bin_path(original_path)


if __name__ == '__main__':
    unittest.main()
